import java.util.Random;
/**
 *
 * File information:<br>
 * $Author: nmk148 $<br>
 * First check-in: $Date: 2003/04/09 16:44:48 $<br>
 * $Revision: 1.3 $<br>
 * <br>
 * This class represents an individual of the population.
 * It has gene information, and can crossover and mutate.
 * Yet, it is an abstract class, since the fitness of an
 * individual depends on the actual problem.
 * The gene needed by the problem is extended by one bit.
 * This (last) bit indicates, whether this individual
 * is age structured or not.
 * 
 **/
public class Individual implements Comparable {

	/* Random number generator for mutation */
	private static Random random = new Random();

	/* The gene information */
	private boolean[] gene;


	/* The problem specification */
	private Problem problem;

	/* Fitness value (if already computed) */
	private double fitness;

	/* Is the fitness value already computed? */
	private boolean fitnessValid;

	/* The number of crossovers an individual may be involved in */
	private int children;

	/* The number of crossovers THIS individual may still be
	 * involved in */
	private int childrenLeft;

	/* The age of this individual (if ASGA)*/
	private int age;

	/* Survival rate at age n */
	private double[] survival;

	/* Is this an asga invidivual */
	private boolean asga;

	
	/** 
	 * Creates a new individual
	 * @param problem the problem specification
	 * @param children the number of children that this
	 *   individual may bear. The result of a crossover
	 *   (on population level) are always two children,
	 *   so technically each individual bears one child
	 *   per crossover.
	 * @param survival survival rates for certain ages
	 * @param asga true, if this should be an ASGA invidivual
	 **/
	public Individual(Problem problem, int children, double[] survival, boolean asga) {
		this.problem = problem;
		gene = new boolean[problem.geneSize()];
		fitnessValid = false;
		this.children = children;
		this.childrenLeft = children;
		this.survival = survival;
		this.asga = asga;
		int age = 0;
	}

	/**
	 * Creates a new individual, eventually with a 
	 * random gene
	 * @param problem the problem specification
	 * @param children the number of children that this
	 *   individual may bear.
	 * @param survival survival rates for certain ages
	 * @param asga true, if this should be an ASGA invidivual
	 * @param r true, if the gene should be initialized
	 *               randomly.
	 **/
	public Individual(Problem problem, int children, 
	       double[] survival, boolean asga, boolean r) {
		this(problem, children,survival,asga);
		if (r) {
			for (int i=0; i<gene.length; i++) {
				gene[i] = random.nextInt(100)<10;
			}
		}
	}

	/**
	 * Clones an individual from another one.
	 **/
	public Individual(Individual ind) {
		this(ind.problem, ind.children, ind.survival, ind.asga);
		for (int i=0; i<ind.gene.length; i++) {
			this.gene[i] = ind.gene[i];
		}
	}
	
	/**
	 * Performs a crossover, where one bit is switched among the 
	 * parents. The whole genome is cloned from this individual,
	 * only the switched bit is taken from the parter.
	 * The number of childrenLeft is decreased in both this and
	 * the other individual.
	 * @param index the index of the bit to be switched
	 * @param asgaSwitch is the asga-bit taken from this of the other 
	 *          individual
	 * @param other the other individual
	 * @return a new GAIndividual that has the same genome as this one,
	 * except for one bit that is taken from the other one.
	 **/
	public Individual crossover(int index, boolean asgaSwitch, Individual other) {
	
		childrenLeft--;
		
		Individual tmp;
		if (asgaSwitch) {
			tmp = new Individual(this.problem, children,
					survival, asga);
		} else {
			tmp = new Individual(this.problem, children,
					survival, other.asga);
		}
		for (int i=0; i<gene.length; i++) {
			tmp.gene[i] = gene[i];
		}
		tmp.gene[index] = other.gene[index];
		return tmp;
	}


	/**
	 * Mutation: Switches one random bit in the genome.
	 **/
	public void mutate() {
		fitnessValid = false;
		int index = random.nextInt(gene.length);
		gene[index] = !gene[index];
	}

	/**
	 * Returns the fitness of this individual
	 * @return the fitness
	 **/
	public double fitness() {
		if (!fitnessValid) {
			fitness = problem.fitness(gene);
		}
		return fitness;
	}


	/**
	 * Increases the age by one. And sets the number of children back to _children_
	 **/
	public void age() {
		childrenLeft = children;
		
		if (asga) age++;
	}

	/**
	 * Checks, whether this individual survives and come into the next generation
	 * @param mate is the individual also mating?
	 * @return true, if the individual survivies
	 **/
	public boolean survive(boolean mate) {
		if (asga) { /* asga -> die by old age */
			return random.nextDouble() < survival[age];
		} else { /* non-asga -> die when mating */
			return !mate;
		}
	}


	/**
	 * Checks, whether this individual has an age structured behavior
	 * @return true, if it is age structured
	 **/
	public boolean asga() {
		return asga;
	}


	/**
	 * Compares this individual to another one based on 
	 * their fitness.
	 * @param o the other individual
	 * @return -1, 0, 1 if this indivdual is smaller, equal or
	 * bigger respectively.
	 **/
	public int compareTo(Object o) {
		if (!(o instanceof Individual)) 
			throw new IllegalStateException("Other no indivdual");

		double fitnessMe = fitness();
		double fitnessOther = ((Individual)o).fitness();


		if (fitnessMe < fitnessOther) return -1;
		if (fitnessMe > fitnessOther) return 1;
		return 0;

	}
	
	 /**
	 * True, if the individual cannot bear any children anymore.
	 * @return true, if there are no children left.
	 **/
	public boolean outOfEnergy() {
		if (Debug.level > 6)
			Debug.assert(childrenLeft>=0,
					"Energy > 0 ("+childrenLeft+")");
		return childrenLeft == 0;
	}

	/** 
	 * Returns a String representation of this Individual.
	 * This includes the actually represented solution.
	 **/
	public String toString() {
		return "(GAInd (fitness=" +fitness()+") ("+
			problem.toString(gene) +"))";
	}
}
