/**
 * File information:<br>
 * $Author: nmk148 $<br>
 * First check-in: $Date: 2003/04/11 17:49:15 $<br>
 * $Revision: 1.6 $<br>
 * <br>
 *
 * This class contains the problem specification for the Traveling
 * Salesperson Problem.
 * <h2>Input</h2>
 * The input for the TSP is a <i>n x n</i>matrix of distances.
 * The number in row <i>i</i> and column <i>j</i> is the distance
 * from city <i>i</i> to city <i>j</i>.<br>
 * <h2>Solution representation</h2>
 * The gene of an individual encodes a solution in the following way:
 * <ul>
 *   <li>A group of <i>ceil(log n)</i> bits in the gene represents 
 *       the number of a city.</li>
 *   <li>The gene is interpreted as an enumeration of such numbers
 *       in the order of the Salesperson traveling through the citys.</li>
 * </ul>
 * The length of the genome is thus <i>ceil(log n)*n</i>
 * <h3>Fitness function</h3>
 * The fitness function has to comply to the following conditions:
 * <ul>
 *    <li>Produce higher values, when the total way is shorter</li>
 *    <li>Give a penalty for cities visited twice.</li>
 *    <li>Give a penalty for unvisited cities.</li>
 * </ul>
 * The last two points are equivalent, so we only need to implement one of
 * them.<br>
 * The penalty has to be big enough to value valid solutions higher than
 * invalid ones, no matter of the total way.<br>
 * We can achieve that by adding two times the biggest value in the matrix 
 * for each city missing in the enumeration.
 **/
public class TSPProblem implements Problem {



	/* Fields */
	public double[][] dist;
	public double biggest;
	public int geneSize;
	public int numberLength;

	/**
	 * Construct a TSP Problem from a distance matrix
	 **/
	public TSPProblem(double[][] dist) {
		this.dist = dist;

		/* Compute genesize */
		/* Compute log n */
		int n = dist.length-1;
		int log = 0;
		while (n>0) {
			log++;
			n /= 2;
		}
		geneSize = dist.length*log;
		numberLength = log;

		/* find biggest number */
		biggest = 0;
		for (int i=0; i<dist.length; i++) {
			for(int j=0; j<dist.length; j++) {
				if (dist[i][j]>biggest)
					biggest = dist[i][j];
			}
		}
	}


	/**
	 * Fitness function as described above
	 **/
	public double fitness(boolean[] gene) {


		int[] path = computePath(gene);
		
		double penalty = penalty(path);
		double totalLen = totalLength(path);


		/**
		 * Fitness value 
		 **/
		return -(totalLen + penalty);
	}


	/**
	 * Compute the sequence of cities represented by 
	 * _gene_. Each city number is encoded in binary
	 * in (log n) consecutive bits.
	 **/
	private int[] computePath(boolean[] gene) {
		int[] path = new int[dist.length];
		/* Compute city enumeration */
		for (int i=0; i<dist.length; i++) {
			int tmp = 0;
			/* little endian */
			for (int j=0; j<numberLength; j++) {
				tmp = 2*tmp+(gene[numberLength*i+j]?1:0);

			}
			path[i] = tmp;
		}
		return path;
	}


	/**
	 * Compute total length of path. 
	 **/
	private double totalLength(int[] path) {
		double tmp = dist[dist.length-1][0];
		for (int i=0; i<path.length-1; i++) {
			tmp += dist[path[i]%dist.length]
				[path[i+1]%dist.length];
		} 
		return tmp;
	}



	/**
	 * Compute penalty for the path.
	 * Each missing city gets a penalty of 2*biggest.
	 **/
	private double penalty(int[] path) {
		
		boolean[] exists = new boolean[dist.length];

		/* Clear _exists_ array */
		for (int i=0; i<exists.length; i++) {
			exists[i] = false;
		}

		/* Remember all visited cities inside _exists_ */
		for (int i=0; i<exists.length; i++) {
			if (path[i] < exists.length) {
				exists[path[i]] = true;
			}
		}

		/* Give a penalty for each nonexisting city */
		double tmp = 0;
		for (int i=0; i<exists.length; i++) {
			if (!exists[i]) {
				tmp += biggest*2.0;
			}

		}
		return tmp;
	}

	/**
	 * The genesize
	 * @return the genesize
	 **/
	public int geneSize() {
		return geneSize;
	}


	/**
	 * Returns the semantics of a gene as a string
	 **/
	public String toString(boolean gene[]) {

		StringBuffer sb = new StringBuffer("Path: ");
		int[] path = computePath(gene);
		for (int i=0; i<path.length; i++) {
			sb = sb.append(path[i]).append(" ");
		}
		sb.append("Length: ");
		sb.append(totalLength(path));
		sb.append(" Penalty: ");
		sb.append(penalty(path));
		return sb.toString();
	}

}
/*
 * History of this file:
 * $Log: TSPProblem.java,v $
 * Revision 1.6  2003/04/11 17:49:15  nmk148
 * Done, handed in everything...
 *
 * Revision 1.5  2003/04/09 16:44:48  nmk148
 * Done testing
 *
 * Revision 1.4  2003/03/28 16:12:22  nmk148
 * Finished
 *
 * Revision 1.3  2003/03/26 18:46:33  nmk148
 * Program done
 *
 * Revision 1.2  2003/03/24 04:40:29  nmk148
 * Replaced GAIndividual and GAPopulation by Individual and Population
 *
 * Revision 1.1  2003/03/23 17:21:26  nmk148
 * First checkin
 *
 */
